package it.eng.spago.cms.operations;

import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.exceptions.BuildOperationException;
import it.eng.spago.cms.constants.Constants;
import it.eng.spago.tracing.TracerSingleton;

/**
 * Models a repository restore operation. The operation has two attributes:
 * <ul>
 * 		<li>path: the path of the cms node (mandatory)</li>
 * 		<li>version: the version of the cms node to restore (mandatory)</li>
 * </ul>
 * The configuration of the operation is stored into a Spago SourceBean structured like below:
 * <pre>
 * &lt;OPERATION name="RESTORE_OBJECT"&gt;
 * 	    &lt;RESTOREOPERATION path="?" version="?" /&gt;  
 * &lt;/OPERATION&gt;
 * <pre>
 */
public class RestoreOperation {

	/**
	 * Internal structure of the operation
	 */
	private SourceBean operationDescriptor = null;
	
	/**
	 * Constructor of the class useful to build an empty restore operation.
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public RestoreOperation() throws BuildOperationException {
		try {
			operationDescriptor = new SourceBean(Constants.NAME_RESTORE_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, "");
			operationDescriptor.setAttribute(Constants.VERSION, "");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "RestoreOperation::<init>(): Error during the creation of the restore operation" +
                                "SourceBean");
			throw new BuildOperationException("RestoreOperation::<init>(): Error during the creation of the " +
					                          "restore operation SourceBean", e);
		}
	}
	
	/**
	 * Constructor of the class useful to build a restore operation
	 * setting the path attribute.
	 * @param path The Path of the cms node 
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public RestoreOperation(String path) throws BuildOperationException {
		try {
			operationDescriptor = new SourceBean(Constants.NAME_RESTORE_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
			operationDescriptor.setAttribute(Constants.VERSION, "");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "RestoreOperation::<init>(String): Error during the creation of the restore operation" +
                                "SourceBean");
			throw new BuildOperationException("RestoreOperation::<init>(String): Error during the creation of the " +
					                          "restore operation SourceBean", e);
		}
	}
	
	
	/**
	 * Constructor of the class useful to build a restore operation
	 * setting the path and version attributes.
	 * @param path The Path of the cms node 
	 * @param version The name of the node version to restore  
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public RestoreOperation(String path, String version) throws BuildOperationException {
		try {
			operationDescriptor = new SourceBean(Constants.NAME_RESTORE_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
			operationDescriptor.setAttribute(Constants.VERSION, version);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "RestoreOperation::<init>(String, String): " +
                                "Error during the creation of the restore operation" +
                                "SourceBean");
			throw new BuildOperationException("RestoreOperation::<init>(String, String): " +
											  "Error during the creation of the " +
					                          "restore operation SourceBean", e);
		}
	}
	
	/**
	 * Gets the Internal SourceBean which defines the internal structure
	 * of the operation
	 * @return The Internal SourceBean of the operation which defines
	 * the internal structure of the operation
	 */
	public SourceBean getOperationDescriptor() {
		return operationDescriptor;
	}

	/**
	 * Set the path of the cms node
	 * @param path Path of the cms node
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setPath(String path) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.PATH, path);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "RestoreOperation::setPath: Error during the path setting");
            throw new BuildOperationException("RestoreOperation::setPath: Error during the path setting", e);
		}
	}
	
	/**
	 * Set the version of the cms node
	 * @param version Version of the cms node
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setVersion(String ver) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.VERSION, ver);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "RestoreOperation::setVersion: Error during the vversion setting");
            throw new BuildOperationException("RestoreOperation::setVersion: Error during the version setting", e);
		}
	}
	
	
}
